var 
c       // (1)
r       // (2)
pi      // (3)
p       // (4)
mc_r    // (5)
g       // (6)
y       // (7)
s       // (8)
y_st    // (9)
lopg    // (10)
pif     // (11)
pf      // (12)
pih     // (13)
ph      // (14)
e       // (15)
pi_st   // (16)
p_st    // (17)
r_st    // (18)
rp      // (19)
q       // (20)
phi
dz      // (21)
z       // (22)
ch      // (23)
ch_st   // (24)
a       // (25)
ac      // (26)
gt      // (27)
b       // (28)
fx      // (29)
gdp     // (30)
cp

// Measurement EQ
R_US    // (31)
PI_US   // (32)
YGR_US  // (33)

R_KOR   // (34)
PI_KOR  // (35)
YGR_KOR // (36)
ds      // (37)
de      // (38)
dFX_KOR // (39)
da
;

// Exogenous variables
varexo 
eps_g % preference shock
eps_dz % technology shock
eps_rp % risk premium shock
eps_r % monetary policy shock
eps_pst % foreign inflation shock
eps_rst % foreign interest shock
eps_yst % foreign output shock
eps_gg
eps_fx
eps_cp

; 

// Parameters
parameters 
hh          % (1)  Habit
sigmaa      % (2)  Inverse intertemporal elasticity of substitution
etaa        % (4)  Elasticity between Home and Foreign Goods
delta_h     % (5)  Indexation rule for domestic firms
theta_h     % (6)  Calvo parameter for domestic firms
phii        % (8)  Inverse Frisch
delta_f     % (9)  Indexation for Retail firms
theta_f     % (10) Calov parameter for retial firms
mp_pi       % (12) Response of policy rate to CPI inflation 
mp_y        % (13) Response of policy rate to output
mp_dy       % (14) Response of policy rate to output growth
mp_de       % (15) Response of policy rate to exchange rate depreciation
rho_g       % (16) Persistence of preference shock
rho_dz       % (17) Persistence of technology shock
rho_rp      % (19) Persistence of risk-premium shock
rho_r       % (20) Persistence of monetary policy shock
rho_gg
rho_cp

rho_fx
beta_fx

betaa       % (7)  Time discount factor (calibrated)
chii
alphaa      % (3)  Openness (calibrated)
nfa_p       % (calibrated)
nfa_g       % (calibrated)
alphaa_st   % (calibrated)
c_y         % (calibrated)

yst_y       % (calibrated)
g_y         % (calibrated)
ch_y        % (calibrated)
chst_y      % (calibrated)
yh_y        % (calibrated)

// Foreign Block : VAR (2) Coefficients

bpp1
bpr1 
bpy1
bpp2
bpr2
bpy2
byp1
byr1
byy1
byp2
byr2
byy2
brp1
brr1
bry1
brp2
brr2
bry2
;


// Calibration
betaa = 0.99 ;      % #1
chii = 0.2 ;        % #2
alphaa = 0.185 ;    % #3 
nfa_p = -0.08 ;
nfa_g = 0.24;
alphaa_st = 0.0093 ; 
c_y = 0.75 ;

yst_y = (1/alphaa_st)*(nfa_p + nfa_g + alphaa*c_y - (1/betaa)*(nfa_p + nfa_g)) ;
g_y = 1 - c_y - alphaa_st*yst_y + alphaa*c_y ;
ch_y = (1-alphaa)*c_y ; 
chst_y = alphaa_st*yst_y ; 
yh_y = ch_y + chst_y + g_y ;


model(linear);
// (1) IS-equation 
c - hh*c(-1) = c(+1) - hh*c - ((1-hh)/sigmaa)*(r - pi(+1)) + ((1-hh)/sigmaa)*(g-g(+1)) + ((1-hh)/sigmaa)*rho_dz*dz; 

// (2) Preference shock
g = rho_g*g(-1) + eps_g ; 

// (3) Technology shock
dz = rho_dz*dz(-1) + eps_dz ;

// (4) Technology difference 
dz = z - z(-1) ;

// (5) Real exchange rate
q = e + p_st - p ;

// (6) Law of One Price Gap
lopg = e + p_st - pf ;

// (7) Terms of Trade
s = pf - ph ;

// (8) Composite price index
p = (1-alphaa)*ph + alphaa*pf ;

// (9) Real marginal cost
mc_r = phii*y + alphaa*s + (sigmaa/(1-hh))*(c - hh*c(-1)) ; 

// (10) Domestic producer's PC
pih - delta_h*pih(-1) = (((1-theta_h)*(1-betaa*theta_h))/theta_h)*mc_r + betaa*(pih(+1) - delta_h*pih) ; 

// (11) Retailer's PC
pif - delta_f*pif(-1) = (((1-theta_f)*(1-betaa*theta_f))/theta_f)*(lopg + cp) + betaa*(pif(+1) - delta_f*pif) ; 

cp = rho_cp*cp(-1) + eps_cp ;


// (12) Monetary policy rule
r = rho_r*r(-1) + (1-rho_r)*(mp_pi*pi + mp_y*gdp) + eps_r ; 

// (13) Foreign CPI inflation
pi_st = bpp1*pi_st(-1) + bpr1*r_st(-1) + bpy1*y_st(-1) + bpp2*pi_st(-2) + bpr2*r_st(-2) + bpy2*y_st(-2) + eps_pst; 

// (14) Foreign output
y_st = byp1*pi_st(-1) +  byr1*r_st(-1) + byy1*y_st(-1) + byp2*pi_st(-2) + byr2*r_st(-2) + byy2*y_st(-2) + eps_yst ; 

// (15) Foreign interest rate
r_st = brp1*pi_st(-1) +  brr1*r_st(-1) + bry1*y_st(-1) + brp2*pi_st(-2) + brr2*r_st(-2) + bry2*y_st(-2) + eps_rst ; 

// (16) Foreign composite price
pi_st = p_st - p_st(-1) ; 

// (17) Domestic price
pih = ph - ph(-1) ;

// (18) Import price
pif = pf - pf(-1) ;

// (19) CPI inflation
pi = p - p(-1) ;

// (20) UIP
r - r_st = e(+1) - e + phi ;

// (21) Risk Premium
phi = - chii*nfa_p*a + rp ;

// (22) Risk premium shock
rp = rho_rp*rp(-1) + eps_rp ; 

// (23) BOP identity
nfa_p*a + nfa_g*ac = alphaa_st*yst_y*(q -(1-etaa)*(s+lopg) + y_st) -alphaa*c_y*(lopg + (1-etaa)*(pf-p) + c) + nfa_p*(1/betaa)*(a(-1) + de -dz - pi + r_st(-1) + phi(-1)) + nfa_g*(1/betaa)*(ac(-1) + de -dz - pi + r_st(-1) + phi(-1)) ;

// (24)
ac = e + fx - p - z; 

// (25)
a = e + b - p - z;

// (26) Definition of GDP
gdp = c_y*c + g_y*gt + alphaa_st*yst_y*(q - (1-etaa)*(s + lopg) + y_st) - alphaa*c_y*(lopg + (1-etaa)*(pf - p) + c)  ;

// (27) Goods Market Clearing 
yh_y*y = ch_y*ch + chst_y*ch_st + g_y*gt ; 

// (28) Domestic demand for domestically produced good
ch = alphaa*etaa*s + c ;

// (29) Foreign demand for domestically produced good
ch_st = etaa*(s + lopg) + y_st;

// (30)
gt = rho_gg*gt(-1) + eps_gg ; 

// (31) FX-rule
fx = rho_fx*fx(-1) - beta_fx*de + eps_fx ; 
//fx - fx(-1) = rho_fx*(fx(-1) - fx(-2)) - beta_fx*de + eps_fx ; 



// Measurement equations (32) - (39) 
R_US = 4*r_st ;
PI_US = 4*pi_st ; 
YGR_US = y_st - y_st(-1) + dz;
de = e - e(-1) ;
ds = s - s(-1) ;

R_KOR = 4*r ;
PI_KOR = 4*pi ;
YGR_KOR = gdp - gdp(-1) + dz ; 
dFX_KOR = fx - fx(-1) + dz ; 
da = a - a(-1) + dz;

end;



estimated_params;
//                 STARTING VALUES &          PRIORS
    sigmaa,    , 1E-5,   ,      GAMMA_PDF,  1.20,    0.40 ;
    phii,      , 1E-5,   ,      GAMMA_PDF,  1.50,    0.75 ;
    theta_h,   , 1E-5,   0.999,      BETA_PDF,   0.50,    0.10 ;
    theta_f,   , 1E-5,   0.999,      BETA_PDF,   0.50,    0.10 ;
    etaa,      , 1E-5,   ,      GAMMA_PDF,  1.50,    0.75 ;
    hh,        , 1E-5,   ,      BETA_PDF,   0.50,    0.25 ;
    delta_h,   , 0,   1,      BETA_PDF,   0.50,    0.25 ;
    delta_f,   , 0,   1,      BETA_PDF,   0.50,    0.25 ;

    rho_r,     , ,   ,      BETA_PDF,   0.50,    0.25 ;
    mp_pi,     , 1E-5,   ,      GAMMA_PDF,  1.50,    0.30 ;
    mp_y,      , 1E-5,   ,      GAMMA_PDF,  0.25,    0.13 ;
//    mp_de,     , 1E-5,   ,      GAMMA_PDF,  0.25,    0.13 ;

    rho_fx,    , ,   ,      BETA_PDF,   0.50,    0.25 ;
    beta_fx,   0.3, ,   ,      GAMMA_PDF,   0.25,    0.13 ;

    rho_dz,    , 1E-10,  0.999,      BETA_PDF,   0.60,    0.2;
    rho_g,     0.8, 1E-10,  0.999,      BETA_PDF,   0.60,    0.2;
    rho_rp,    , 1E-10,  0.999,      BETA_PDF,   0.60,    0.2;
    rho_gg,    , 1E-10,  0.999,      BETA_PDF,   0.60,    0.2;
    rho_cp,    , 1E-10,  0.999,      BETA_PDF,   0.60,    0.2;


// First order Authoregressive
    bpp1,      ,      ,   ,      NORMAL_PDF, 0.6,    0.20;
    byy1,      ,      ,   ,      NORMAL_PDF, 0.6,    0.20;
    brr1,      ,      ,   ,      NORMAL_PDF, 0.6,    0.20;

// Second order Own lag
    bpp2,      ,      ,   ,      NORMAL_PDF, 0.00,    0.2;
    byy2,      ,      ,   ,      NORMAL_PDF, 0.00,    0.2;
    brr2,      ,      ,   ,      NORMAL_PDF, 0.00,    0.2;

// Off-diagonal elements of the first lag
    bpr1,      ,      ,   ,      NORMAL_PDF, 0.00,    0.20;
    bpy1,      ,      ,   ,      NORMAL_PDF, 0.00,    0.20;
    byp1,      ,      ,   ,      NORMAL_PDF, 0.00,    0.20;
    byr1,      ,      ,   ,      NORMAL_PDF, 0.00,    0.20;
    brp1,      ,      ,   ,      NORMAL_PDF, 0.00,    0.20;
    bry1,      ,      ,   ,      NORMAL_PDF, 0.00,    0.20;

// Off-diagonal elements of the second lag
    bpr2,      ,      ,   ,      NORMAL_PDF, 0.00,    0.2;
    bpy2,      ,      ,   ,      NORMAL_PDF, 0.00,    0.2;
    byp2,      ,      ,   ,      NORMAL_PDF, 0.00,    0.2;
    byr2,      ,      ,   ,      NORMAL_PDF, 0.00,    0.2;
    brp2,      ,      ,   ,      NORMAL_PDF, 0.00,    0.2;
    bry2,      ,      ,   ,      NORMAL_PDF, 0.00,    0.2;

// Shocks
    stderr eps_dz,   ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;
    stderr eps_g,    ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;
    stderr eps_rp,   ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;
    stderr eps_r,    ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;
    stderr eps_pst,  ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;
    stderr eps_rst,  ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;
    stderr eps_yst,  ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;
    stderr eps_gg,   ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;
    stderr eps_fx,   ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;
    stderr eps_cp,   ,    1E-10,   ,      INV_GAMMA1_PDF,0.5,inf;

end;

varobs PI_US YGR_US R_US PI_KOR R_KOR YGR_KOR de ds dFX_KOR da;

//estimation(datafile=data_neer_jp2,plot_priors=0,nograph,mode_compute=0,mode_check,prefilter=1,first_obs=33,nobs=30,mh_nblocks=1,diffuse_filter,mh_replic=50000,mh_jscale=0.3);
estimation(datafile=data_neer_jp2,mode_file=fx_est_post2_mode, plot_priors=0,nograph,mode_compute=0,mode_check,prefilter=1,first_obs=33,nobs=30,mh_nblocks=1,diffuse_filter,mh_replic=50000,mh_jscale=0.3,bayesian_irf,irf=12,moments_varendo);
